@@ -16,9 +16,9 @@ from jinja2 import Environment, FileSystemLoader
 from .logger import get_cfy_cluster_manager_logger, setup_logger
 from .utils import (check_cert_key_match, check_cert_path, check_san,
                     check_signed_by, cloudify_rpm_is_installed,
-                    ClusterInstallError, copy, current_host_ip,
-                    get_dict_from_yaml, move, raise_errors_list, run,
-                    sudo, VM, write_dict_to_yaml_file, yum_is_present)
+                    ClusterInstallError, copy, get_dict_from_yaml, move,
+                    raise_errors_list, run, sudo, VM,
+                    write_dict_to_yaml_file, yum_is_present)
 
 logger = get_cfy_cluster_manager_logger()
 
@@ -29,8 +29,8 @@ DIR_NAME = 'cloudify_cluster_manager'
 RPM_NAME = 'cloudify-manager-install.rpm'
 TOP_DIR = '/tmp'
 
-RPM_PATH = join(TOP_DIR, RPM_NAME)
 CLUSTER_INSTALL_DIR = join(TOP_DIR, DIR_NAME)
+RPM_PATH = join(CLUSTER_INSTALL_DIR, RPM_NAME)
 CERTS_DIR = join(CLUSTER_INSTALL_DIR, CERTS_DIR_NAME)
 CONFIG_FILES_DIR = join(CLUSTER_INSTALL_DIR, CONFIG_FILES)
 
@@ -55,13 +55,14 @@ class CfyNode(VM):
                  public_ip,
                  key_file_path,
                  username,
+                 password,
                  node_name,
                  hostname,
                  cert_path,
                  key_path,
                  config_file_path):
         super(CfyNode, self).__init__(private_ip, public_ip,
-                                      key_file_path, username)
+                                      key_file_path, username, password)
         self.name = node_name
         self.hostname = hostname
         self.provided_cert_path = expanduser(cert_path) if cert_path else None
@@ -242,11 +243,7 @@ def _prepare_config_files(instances_dict, credentials, config):
     )
 
 
-def _install_cloudify_remotely(instance, rpm_download_link):
-    logger.info('Downloading Cloudify RPM on %s from %s',
-                instance.name, rpm_download_link)
-    instance.run_command('curl -o {0} {1}'.format(
-        RPM_PATH, rpm_download_link), hide_stdout=True)
+def _install_cloudify_remotely(instance):
     logger.info('Installing Cloudify RPM on %s', instance.name)
     instance.run_command(
         'yum install -y {}'.format(RPM_PATH), use_sudo=True, hide_stdout=True)
@@ -345,13 +342,9 @@ def _verify_cloudify_installed_successfully(instance):
             'Service {} status is unknown'.format(instance.unit_name))
 
 
-def _install_instances(instances_dict,
-                       using_three_nodes,
-                       rpm_download_link,
-                       verbose):
+def _install_instances(instances_dict, verbose, override):
     for i, instance_type in enumerate(instances_dict):
         logger.info('installing %s instances', instance_type)
-        three_nodes_not_first_round = using_three_nodes and i > 0
         for instance in instances_dict[instance_type]:
             if instance.installed:
                 logger.info('Already installed %s (%s)',
@@ -359,18 +352,10 @@ def _install_instances(instances_dict,
                 continue
 
             logger.info('Installing %s', instance.name)
-            if (instance.private_ip == current_host_ip() or
-                    three_nodes_not_first_round):
-                logger.info('Already prepared the cluster install files on '
-                            'this instance')
-            else:
-                logger.info('Copying the %s directory to %s',
-                            CLUSTER_INSTALL_DIR, instance.name)
-                instance.put_dir(CLUSTER_INSTALL_DIR,
-                                 CLUSTER_INSTALL_DIR,
-                                 override=True)
-                if not _rpm_was_installed(instance):
-                    _install_cloudify_remotely(instance, rpm_download_link)
+            instance.put_dir(CLUSTER_INSTALL_DIR, CLUSTER_INSTALL_DIR)
+
+            if not _rpm_was_installed(instance):
+                _install_cloudify_remotely(instance)
 
             instance.run_command('cp {0} {1}'.format(
                 join(CONFIG_FILES_DIR, '{}_config.yaml'.format(instance.name)),
@@ -406,6 +391,7 @@ def _get_cfy_node(config, node_dict, node_name, config_path,
                      node_dict.get('public_ip'),
                      config.get('ssh_key_path'),
                      config.get('ssh_user'),
+                     config.get('ssh_password'),
                      node_name,
                      node_dict.get('hostname'),
                      cert_path=node_dict.get('cert_path'),
@@ -514,14 +500,18 @@ def _print_successful_installation_message(start_time, msg='installed'):
                 '{1} minutes and {2} seconds'.format(msg, int(m), int(s)))
 
 
-def _install_cloudify_locally(rpm_download_link):
+def _install_cloudify_locally(rpm_path):
     if cloudify_rpm_is_installed():
         logger.info('Cloudify RPM is already installed')
+    expanded_rpm_path = expanduser(rpm_path)
+    if exists(expanded_rpm_path):
+        copy(expanded_rpm_path, RPM_PATH)
     else:
-        logger.info('Downloading Cloudify RPM from %s', rpm_download_link)
-        run(['curl', '-o', RPM_PATH, rpm_download_link])
+        logger.info('Downloading Cloudify RPM from %s', rpm_path)
+        run(['curl', '-o', RPM_PATH, rpm_path])
         logger.info('Installing Cloudify RPM')
-        sudo(['yum', 'install', '-y', RPM_PATH])
+
+    sudo(['yum', 'install', '-y', RPM_PATH])
 
 
 def _check_path(dictionary, key, errors_list, vm_name=None):
@@ -570,7 +560,7 @@ def _validate_config_paths(existing_vms_dict, using_three_nodes, errors_list):
                 errors_list.append(
                     'Certificate can not be specified for {0} because a '
                     'config path was specified for it. If you wish to use '
-                    'your own config path, plase make sure the relevant '
+                    'your own config path, please make sure the relevant '
                     'certificates are on each VM.'.format(vm_name))
             if using_three_nodes:
                 for config_name, config_path in vm_dict['config_path'].items():
@@ -661,12 +651,23 @@ def _validate_ldap_certificate_setting(config, errors_list):
         _check_path(config.get('ldap'), 'ca_cert', errors_list)
 
 
+def _validate_ssh_config(config, errors_list):
+    ssh_key_path = config.get('ssh_key_path')
+    ssh_password = config.get('ssh_password')
+    if (not ssh_key_path and not ssh_password) or \
+            (ssh_key_path and ssh_password):
+        errors_list.append(
+            'Please provide only one of ssh_key_path or ssh_password')
+    if config.get('ssh_key_path'):
+        _check_path(config, 'ssh_key_path', errors_list)
+    _check_value_provided(config, 'ssh_user', errors_list)
+
+
 def validate_config(config, using_three_nodes_cluster, override):
     errors_list = []
-    _check_path(config, 'ssh_key_path', errors_list)
-    _check_value_provided(config, 'ssh_user', errors_list)
+    _validate_ssh_config(config, errors_list)
     _check_path(config, 'cloudify_license_path', errors_list)
-    _check_value_provided(config, 'manager_rpm_download_link', errors_list)
+    _check_value_provided(config, 'manager_rpm_path', errors_list)
     _validate_existing_vms(config, using_three_nodes_cluster, errors_list)
     _validate_external_db_config(config, override, errors_list)
     _validate_ldap_certificate_setting(config, errors_list)
@@ -832,6 +833,8 @@ def _remove_cloudify_installation(instance, verbose):
                 instance.run_command(
                     'mv {0} {1}'.format(full_path, new_path), use_sudo=True)
 
+    instance.run_command('rm -rf {}'.format(CLUSTER_INSTALL_DIR))
+
     instance.installed = False
 
 
@@ -902,7 +905,6 @@ def install(config_path, override, only_validate, verbose):
         logger.info('The configuration file at %s was validated '
                     'successfully.', config_path)
         return
-    rpm_download_link = config.get('manager_rpm_download_link')
     instances_dict = (_generate_three_nodes_cluster_dict(config)
                       if using_three_nodes_cluster else
                       _generate_general_cluster_dict(config))
@@ -912,18 +914,17 @@ def install(config_path, override, only_validate, verbose):
         logger.info('Cloudify cluster was previously installed')
         _handle_installed_instances(instances_dict, override, verbose)
     if (not previous_installation) or override:
-        logger.info('Preparing cluster mangaer files')
+        logger.info('Preparing cluster manager files')
         _create_cluster_install_directory()
         copy(config.get('cloudify_license_path'),
              join(CLUSTER_INSTALL_DIR, 'license.yaml'))
-        _install_cloudify_locally(rpm_download_link)
+        _install_cloudify_locally(config.get('manager_rpm_path'))
         if not _using_provided_config_files(instances_dict):
             _handle_certificates(config, instances_dict)
             credentials = _handle_credentials(config.get('credentials'))
         _prepare_config_files(instances_dict, credentials, config)
 
-    _install_instances(instances_dict, using_three_nodes_cluster,
-                       rpm_download_link, verbose)
+    _install_instances(instances_dict, verbose, override)
     _log_managers_connection_strings(instances_dict['manager'])
     if credentials:
         logger.warning('The credentials file was saved to %s. '
